""" IDA ROP view plugin UI functions and classes """

# Python libraries
import os
import csv
import logging

# IDA libraries
import idaapi
import idc
from idaapi import Form, Choose, Choose2

# Choose2 has disappeared from IdaPython v7
if idaapi.IDA_SDK_VERSION <= 695:
    from idaapi import Choose2 as SegmentChoose
elif idaapi.IDA_SDK_VERSION >= 700:
    import ida_idaapi
    from idaapi import Choose as SegmentChoose
else:
    pass

# IDA plugin
try :
    import netnode
    netnode_package = True
except ImportError as ie:
    netnode_package =  False
    

from .engine import IdaRopEngine, IdaRopSearch, Gadget


class IdaRopForm(Form):
    """ Ida Rop Search input form """

    def __init__(self, idaropengine,  select_list = None):

        self.engine = idaropengine
        self.select_list = select_list
        self.segments = SegmentView(self.engine)


        Form.__init__(self, 
r"""BUTTON YES* Search
Search ROP gadgets

{FormChangeCb}<Segments:{cEChooser}>

<Bad Chars        :{strBadChars}>     
Unicode Table    <ANSI:{rUnicodeANSI}><OEM:{rUnicodeOEM}><UTF7:{rUnicodeUTF7}><UTF8:{rUnicodeUTF8}>{radUnicode}>

<Max gadget size  :{intMaxRopSize}>      
<Max gadget offset:{intMaxRopOffset}>       <Search for ROP gadgets:{cRopSearch}>
<Max RETN imm16   :{intMaxRetnImm}>         <Search for JOP gadgets:{cJopSearch}>
<Max gadgets      :{intMaxRops}>            <Search for SYS gadgets:{cSysSearch}>{gadgetGroup}>

Others settings:
<Allow conditional jumps:{cRopAllowJcc}> <Do not allow bad bytes:{cRopNoBadBytes}>{ropGroup}>
""", {
                'cEChooser'       : Form.EmbeddedChooserControl(self.segments, swidth=110),
                'ropGroup'        : Form.ChkGroupControl(('cRopAllowJcc','cRopNoBadBytes')),
                'gadgetGroup'     : Form.ChkGroupControl(('cRopSearch','cJopSearch','cSysSearch')),
                'intMaxRopSize'   : Form.NumericInput(swidth=4,tp=Form.FT_DEC,value=self.engine.rop.maxRopSize),
                'intMaxRopOffset' : Form.NumericInput(swidth=4,tp=Form.FT_DEC,value=self.engine.rop.maxRopOffset),
                'intMaxRops'      : Form.NumericInput(swidth=4,tp=Form.FT_DEC,value=self.engine.rop.maxRops),
                'intMaxRetnImm'   : Form.NumericInput(swidth=4,tp=Form.FT_HEX,value=self.engine.rop.maxRetnImm),
                'intMaxJopImm'    : Form.NumericInput(swidth=4,tp=Form.FT_HEX,value=self.engine.rop.maxJopImm),
                'strBadChars'     : Form.StringInput(swidth=92,tp=Form.FT_ASCII),
                'radUnicode'      : Form.RadGroupControl(("rUnicodeANSI","rUnicodeOEM","rUnicodeUTF7","rUnicodeUTF8")),
                'strBadMnems'     : Form.StringInput(swidth=92,tp=Form.FT_ASCII,value="into, in, out, loop, loope, loopne, lock, rep, repe, repz, repne, repnz"),
                'FormChangeCb'    : Form.FormChangeCb(self.OnFormChange),
            })

        self.Compile()

    def OnFormChange(self, fid):

        # Form initialization
        if fid == -1:
            self.SetFocusedField(self.cEChooser)

            # Preselect executable segments on startup if none were already specified:
            if self.select_list == None:

                self.select_list = list()

                for i, seg in enumerate(self.engine.segments):
                    if seg.x:
                        self.select_list.append(i)

            self.SetControlValue(self.cEChooser, self.select_list)

            # Enable both ROP and JOP search by default
            self.SetControlValue(self.cRopSearch, True)
            self.SetControlValue(self.cJopSearch, True)
            self.SetControlValue(self.cSysSearch, False)

            # Skip bad instructions by default
            self.SetControlValue(self.cRopNoBadBytes, True)

        # Form OK pressed
        elif fid == -2:
            pass

        return 1

###############################################################################
class SegmentView(SegmentChoose):

    def __init__(self, idarop):

        self.idarop = idarop

        SegmentChoose.__init__(self, "Segments",
                         [ ["Name",   13 | SegmentChoose.CHCOL_PLAIN],
                           ["Start",  13 | SegmentChoose.CHCOL_HEX], 
                           ["End",    10 | SegmentChoose.CHCOL_HEX], 
                           ["Size",   10 | SegmentChoose.CHCOL_HEX],
                           ["R",       1 | SegmentChoose.CHCOL_PLAIN],
                           ["W",       1 | SegmentChoose.CHCOL_PLAIN], 
                           ["X",       1 | SegmentChoose.CHCOL_PLAIN],
                           ["Class",   8 | SegmentChoose.CHCOL_PLAIN], 
                         ],
                         flags = SegmentChoose.CH_MULTI,  # Select multiple modules
                         embedded=True)

        self.icon = 150

        # Items for display
        self.items = list()

        # Selected items
        self.select_list = list()

        # Initialize/Refresh the view
        self.refreshitems()

    def OnSelectionChange(self, selection_list):
        # "Temporary" crutch since GetEmbSelection() does not work for Ida 7.0
        self.select_list = selection_list

    def show(self):
        # Attempt to open the view
        if self.Show() < 0: return False

    def refreshitems(self):
        self.items = list()

        for segment in self.idarop.list_segments():
            self.items.append(segment.get_display_list())

    def OnCommand(self, n, cmd_id):

        # Search ROP gadgets
        if cmd_id == self.cmd_search_gadgets:
            
            # Initialize ROP gadget form with empty selection
            self.idarop.process_rop(select_list = self.select_list)

    def OnSelectLine(self, n):
        pass

    def OnGetLine(self, n):
        return self.items[n]

    def OnGetIcon(self, n):


        if not len(self.items) > 0:
            return -1

        segment = self.idarop.list_segments()[n]

        if segment.x : # Exec Seg
            return 61
        else:
            return 59

    def OnClose(self):
        self.cmd_search_gadgets = None

    def OnGetSize(self):
        return len(self.items)

    def OnRefresh(self, n):
        self.refreshitems()
        return n

    def OnActivate(self):
        self.refreshitems()


class IdaRopView(Choose2):
    """
    Chooser class to display security characteristics of loaded modules.
    """
    def __init__(self, idarop):

        self.idarop = idarop

        Choose2.__init__(self,
                         "ROP gadgets",
                         [ ["Segment",           13 | Choose2.CHCOL_PLAIN],
                           ["Address",           13 | Choose2.CHCOL_HEX],
                           ["Return Address",    13 | Choose2.CHCOL_HEX], 
                           ["Gadget",            30 | Choose2.CHCOL_PLAIN], 
                           ["Opcodes",           20 | Choose2.CHCOL_PLAIN],
                           ["Size",               3 | Choose2.CHCOL_DEC],
                           ["Pivot",              4 | Choose2.CHCOL_DEC],
                         ],
                         flags = Choose2.CH_MULTI)

        self.icon = 182

        # Items for display
        self.items = []

        # rop list cache for instantaneous loading if there has not been any new data
        self.rop_list_cache = None

        # Initialize/Refresh the view
        self.refreshitems()

        # export as csv command
        self.cmd_export_csv  = None

        # clear result command
        self.clear_rop_list = None

        

    def show(self):
        # Attempt to open the view
        if self.Show() < 0: return False

        if self.cmd_export_csv == None:
            self.cmd_export_csv  = self.AddCommand("Export as csv...", flags = idaapi.CHOOSER_POPUP_MENU, icon=40)
        if self.clear_rop_list == None:
            self.clear_rop_list  = self.AddCommand("Clear rop list", flags = idaapi.CHOOSER_POPUP_MENU, icon=32)

        return True

    def refreshitems(self):

        # Pb : rop engine has not been init
        if self.idarop.rop == None:
            return 

        # No new data present
        if self.rop_list_cache == self.idarop.rop.gadgets:
            return

        self.items = []

        # No data present
        if len(self.idarop.rop.gadgets) == 0:
            return


        if len(self.idarop.rop.gadgets) > 10000:
            idaapi.show_wait_box("Ida rop : loading rop list ...")

        for i,g in enumerate(self.idarop.rop.gadgets):

            # reconstruct disas
            if g.opcodes == "":

                bad_gadget = False
                opcodes = idc.GetManyBytes(g.address, g.ret_address - g.address + 1)
                instructions = list()
                ea = g.address
                while ea <= g.ret_address:
                    instructions.append(idc.GetDisasmEx(ea, idaapi.GENDSM_FORCE_CODE))
                    ea += idaapi.decode_insn(ea) 

                    # Badly decoded gadget
                    if idaapi.decode_insn(ea) == 0:
                        bad_gadget = True
                        break


                if not bad_gadget:
                    h = Gadget(
                        address = g.address,
                        ret_address = g.ret_address,
                        instructions = instructions,
                        opcodes = opcodes,
                        size = len(opcodes)
                    )
                    self.idarop.rop.gadgets[i] = h

                    self.items.append(h.get_display_list(self.idarop.addr_format))
            else:
                self.items.append(g.get_display_list(self.idarop.addr_format))

        self.rop_list_cache = self.idarop.rop.gadgets
        if len(self.idarop.rop.gadgets) > 10000:
            idaapi.hide_wait_box()



    def OnCommand(self, n, cmd_id):

        # Export CSV
        if cmd_id == self.cmd_export_csv:

            file_name = idaapi.askfile_c(1, "*.csv", "Please enter CSV file name")
            if file_name:
                print ("[idarop] Exporting gadgets to %s" % file_name)
                with open(file_name, 'wb') as csvfile:
                    csvwriter = csv.writer(csvfile, delimiter=',',
                                            quotechar='"', quoting=csv.QUOTE_MINIMAL)
                    csvwriter.writerow(["Address","Gadget","Size","Pivot"])
                    for item in self.items:
                        csvwriter.writerow(item)

        elif cmd_id == self.clear_rop_list:
            self.idarop.clear_rop_list()
            self.refreshitems()

        return 1

    def OnSelectLine(self, n):
        """ Callback on double click line : should open a custom view with the disas gadget.
            IDA disass view can't show "unaligned" gadgets.
        """
        idaapi.jumpto( self.idarop.rop.gadgets[n].address )

    def OnGetLine(self, n):
        return self.items[n]

    def OnClose(self):
        self.cmd_export_csv  = None
        self.clear_rop_list  = None

    def OnGetSize(self):
        return len(self.items)

    def OnRefresh(self, n):
        self.refreshitems()
        return n

    def OnActivate(self):
        self.refreshitems()


if idaapi.IDA_SDK_VERSION >= 700:
    class SearchGadgetsHandler(idaapi.action_handler_t):
            def __init__(self, manager):
                idaapi.action_handler_t.__init__(self)
                self._manager = manager

            def activate(self, ctx):
                self._manager.proc_rop()
                return 1

            def update(self, ctx):
                return idaapi.AST_ENABLE_ALWAYS

    class ListGadgetsHandler(idaapi.action_handler_t):
            def __init__(self, manager):
                idaapi.action_handler_t.__init__(self)
                self._manager = manager

            def activate(self, ctx):
                self._manager.show_rop_view()
                return 1

            def update(self, ctx):
                return idaapi.AST_ENABLE_ALWAYS
else:
    pass

class IdaRopManager():
    """ Top-level object managing IDA Rop View plugin """

    def __init__(self): 
 
        # Initialize ROP gadget search engine
        self.engine = IdaRopEngine()
        self.engine.rop = IdaRopSearch(self.engine)
        self.ropView = IdaRopView(self.engine)

        # Defered csv loading for a faster startup
        self.defered_loading = False

        # List of menu item added by the plugin
        self.addmenu_item_ctxs = list()

        # blob manager for saving internal db into idb file
        self.blob_manager = None
        if netnode_package :
            self.blob_manager = netnode.Netnode("$ idarop.rop_blob")
        else:
            print("[IdaRop] IdaRop rely on the Netnode package to save the rop database in the idb file.")
            print("         Since it's not present, the results will be discarded when closing IDA.")
    

    def add_menu_items(self):
        """ Init additions to Ida's menu entries """

        if idaapi.IDA_SDK_VERSION <= 695:
            def add_menu_item_helper(self, menupath, name, hotkey, flags, pyfunc, args):
                """ helper for adding a menu item  """

                # add menu item and report on errors
                addmenu_item_ctx = idaapi.add_menu_item(menupath, name, hotkey, flags, pyfunc, args)
                if addmenu_item_ctx is None:
                    return 1
                else:
                    self.addmenu_item_ctxs.append(addmenu_item_ctx)
                    return 0

            if add_menu_item_helper(self, "Search/all error operands", "list rop gadgets...", "Ctrl+Shift+r", 1, self.proc_rop, None): return 1
            if add_menu_item_helper(self, "View/Open subviews/Problems", "View rop gadgets...", "Shift+r", 1, self.show_rop_view, None): return 1
            return 0

        elif idaapi.IDA_SDK_VERSION >= 700:
            
            search_gadgets_desc = idaapi.action_desc_t(
                'idarop:searchgadgets',             
                'search rop gadgets...',              
                SearchGadgetsHandler(self),         
                "Ctrl+Shift+r",                     
                'search all gadgets available on this binary', 
            ) 
            idaapi.register_action(search_gadgets_desc)

            idaapi.attach_action_to_menu(
                'Search/IdaRop/',
                'idarop:searchgadgets',
                idaapi.SETMENU_APP
            )

            list_gadgets_desc = idaapi.action_desc_t(
                'idarop:listgadgets',             
                'list rop gadgets...',              
                ListGadgetsHandler(self),         
                "Shift+r",                     
                'list all gadgets searched on this binary', 
            ) 
            idaapi.register_action(list_gadgets_desc)

            idaapi.attach_action_to_menu(
                'Search/IdaRop/',
                'idarop:listgadgets',
                idaapi.SETMENU_APP
            )

            return 0
        else:
            return 0

    
    def del_menu_items(self):
        """ Clear Ida Rop plugin menu entries """
        if idaapi.IDA_SDK_VERSION <= 695:
            for addmenu_item_ctx in self.addmenu_item_ctxs:
                idaapi.del_menu_item(addmenu_item_ctx)
        elif idaapi.IDA_SDK_VERSION >= 700:
            idaapi.detach_action_from_menu('Search/IdaRop/', 'idarop:listgadgets')
            idaapi.detach_action_from_menu('Search/IdaRop/', 'idarop:searchgadgets')
        else:
            pass

    
    def show_rop_view(self):
        """ Show the list of rop gadgets found """

        # If the default csv exist but has not been loaded, load here
        if self.defered_loading == True:
            idaapi.show_wait_box("loading gadgets db ...")
            self.load_default_csv(force = True)
            idaapi.hide_wait_box()
            self.defered_loading = False

        # Show the ROP gadgets view
        self.ropView.refreshitems()
        self.ropView.show()

    def proc_rop(self):
        """ Search for rop gadgets, based on user input options """
        
        # Prompt user for ROP search settings
        f = IdaRopForm(self.engine)
        ok = f.Execute()
        if ok == 1:
            # reset previous results
            self.defered_loading = False
            
            select_list = f.segments.select_list
            ret = self.engine.process_rop(f, select_list)

            if ret:
                self.show_rop_view()

                # force redraw of every list views
                idaapi.refresh_lists()

        f.Free()


    def save_internal_db(self):
        """ store the found rop gadget in the default internal db """

        if len(self.engine.rop.gadgets) == 0 or self.blob_manager == None:
            return
        
        cancel_flag = False
        internal_repr = list()
        for item in self.engine.rop.gadgets:

            address,ret_addres = item.address, item.ret_address
            offset = "0x%x" % (address - idaapi.get_imagebase())
            ret_offset = "0x%x" % (ret_addres - idaapi.get_imagebase())

            internal_repr.append((offset, ret_offset))

            if idaapi.wasBreak():
                cancel_flag = True
                print("[IdaRop] save internal db interrupted.")
                break

        # save only on success
        if not cancel_flag:
            txt_repr = ";".join( "%s:%s" % (g[0],g[1]) for g in internal_repr)
            self.blob_manager["db"] = txt_repr

    def load_internal_db(self, force=False):
        """ Load the rop gadgets list from the internal db """

        if self.blob_manager == None :
            return

        internal_repr =  self.blob_manager["db"].split(";")
        if internal_repr == None:
            return

        for item in internal_repr:
            offset,ret_offset = item.split(':')
            
            # Reconstruct linear address based on binary base address and offset
            address = int(offset, 16) + idaapi.get_imagebase()
            ret_address = int(ret_offset, 16) + idaapi.get_imagebase()

            gadget = Gadget(
                address = address,
                ret_address =  ret_address,
                instructions = list(),
                opcodes = "",
                size = 0
            )

            self.engine.rop.gadgets.append(gadget)

            if idaapi.wasBreak():
                print("[IdaRopLoad] Load csv file interrupted.")
                break


""" IDA ROP view plugin rop search engine and processing """

# Python libraries
import binascii
import sys
import logging
from struct import pack, unpack
from collections import namedtuple

# IDA libraries
import idaapi
import idc

if idaapi.IDA_SDK_VERSION <= 695:
    from idaapi import get_segm_qty, getnseg
if idaapi.IDA_SDK_VERSION >= 700:
    from ida_segment import get_segm_qty, getnseg
else:
    pass

###############################################################################
# Data Structure class

class SegmentEntry(namedtuple('Segment','name start end size r w x segclass')):
    """ Segment entry container for listing segments and characteristics  """

    __slots__ = ()

    def get_display_list(self):
        """ Return the display format list for the segment listing """
        return [ self.name ,
                 "%08X" % self.start,
                 "%08X" % self.end,
                 "%08X" % self.size,
                 (".", "R")[self.r],
                 (".", "W")[self.w],
                 (".", "X")[self.x],
                 self.segclass]

class Gadget(namedtuple('Gadget', 'address ret_address instructions opcodes size')):
    """ Gadget element container for rop listing and export to csv """

    __slots__ = ()

    def get_display_list(self, address_format):
        """ Return the display format list for the rop gadget listing """
        txt_instructions = " ; ".join(self.instructions)
        txt_opcodes = " ".join("%02x" % ord(op) for op in self.opcodes)
        return [ idc.SegName(self.address),
                 address_format % self.address, 
                 address_format % self.ret_address, 
                 txt_instructions,
                 txt_opcodes,
                 "%d" % len(self.opcodes), 
                 ("N", "Y")["sp" in txt_instructions]
                 ]



# ROP Search Engine
class IdaRopSearch():

    def __init__(self, sploiter):
        
        self.maxRopOffset = 40 # Maximum offset from the return instruction to look for gadgets. default: 40
        self.maxRopSize   = 6  # Maximum number of instructions to look for gadgets. default: 6
        self.maxRetnImm   = 64 # Maximum imm16 value in retn. default: 64
        self.maxJopImm    = 255 # Maximum jop [reg + IMM] value. default: 64
        self.maxRops      = 0  # Maximum number of ROP chains to find. default: 0 (unlimited)

        self.debug        = False

        self.regnames     = idaapi.ph_get_regnames()

        self.sploiter     = sploiter
        self.retns        = list()
        self.gadgets      = list()

        # Decoded instruction cache
        self.insn_cache = dict()

        # Extra bytes to read to ensure correct decoding of
        # RETN, RETN imm16, CALL /2, and JMP /4 instructions.
        self.dbg_read_extra = 6 # FF + ModR/M + SIB + disp32 

        self.insn_arithmetic_ops = ["inc","dec","neg", "add","sub","mul","imul","div","idiv","adc","sbb","lea"]
        self.insn_bit_ops = ["not","and","or","xor","shr","shl","sar","sal","shld","shrd","ror","rcr","rcl"]

    def get_o_reg_name(self, insn, n):

        reg_num = insn.Operands[n].reg
        reg_name = self.regnames[reg_num]

        # NOTE: IDA's x86/x86-64 regname array contains only register root names
        #       (e.g ax,cx,dx,etc.). However we can still figure out exact register
        #       size by looking at the operand 'dtyp' property.
        if reg_num < 8:

            # 32-bit register
            if insn.Operands[n].dtyp == idaapi.dt_dword:
                reg_name = 'e'+reg_name

            # 64-bit register
            elif insn.Operands[n].dtyp == idaapi.dt_qword:
                reg_name = 'r'+reg_name

            # 16-bit register otherwise

        return reg_name

    def search_retns(self):

        self.retns = list()

        # Iterate over segments in the module
        # BUG: Iterating over all loaded segments is more stable than looking up by address
        for n in self.segments:
            segment = getnseg(n)

            # Locate executable segments in a selected modules
            # NOTE: Each module may have multiple executable segments
            if segment and segment.perm & idaapi.SEGPERM_EXEC:
               
                #######################################################
                # Search for ROP gadgets
                self.search_rop_gadgets(segment, ret_preamble = 0xc3 ) # RETN 
                self.search_rop_gadgets(segment, ret_preamble = 0xcb ) # RETF
                self.search_rop_gadgets(segment, ret_preamble = 0xc2 ) # RETN imm16
                self.search_rop_gadgets(segment, ret_preamble = 0xca ) # RETN imm16
                self.search_rop_gadgets(segment, ret_preamble = 0xf2c3 ) # MPX RETN 
                self.search_rop_gadgets(segment, ret_preamble = 0xf2c2 ) # MPX RETN imm16

                #######################################################
                # Search for JOP gadgets
                self.search_job_gadgets(segment, jump_preamble = 0xff )
                self.search_job_gadgets(segment, jump_preamble = 0xf2ff ) # MPX
                
                #######################################################
                # Search for SYS gadgets
                self.search_sys_gadgets(segment)

        print("[IdaRopSearch] Found %d returns" % len(self.retns))


    def is_job_gadget(self, jop):
        """ jump oriented gadget predicate """

        ###################################################
        # JMP/CALL reg
        if jop[0] in ["\xe0","\xe1","\xe2","\xe3","\xe4","\xe5","\xe6","\xe7",
                      "\xd0","\xd1","\xd2","\xd3","\xd4","\xd5","\xd6","\xd7"]:
            return True

        ###################################################
        # JMP/CALL [reg] no SIB
        # NOTE: Do not include pure [disp] instruction.

        # JMP/CALL [reg] no *SP,*BP
        elif jop[0] in ["\x20","\x21","\x22","\x23","\x26","\x27", 
                        "\x10","\x11","\x12","\x13","\x16","\x17"]:
            return True

        # JMP/CALL [reg + imm8] no *SP
        elif jop[0] in ["\x60","\x61","\x62","\x63","\x65","\x66","\x67",
                        "\x50","\x51","\x52","\x53","\x55","\x56","\x57"]:
            jop_imm8 = jop[1]
            jop_imm8 = unpack("b", jop_imm8)[0] # signed

            if jop_imm8 <= self.maxJopImm:
                return True


        # JMP/CALL [reg + imm32] no *SP
        elif jop[0] in ["\xa0","\xa1","\xa2","\xa3","\xa5","\xa6","\xa7",
                        "\x90","\x91","\x92","\x93","\x95","\x96","\x97"]:
            jop_imm32 = jop[1:5]
            jop_imm32 = unpack("<i", jop_imm32)[0] # signed

            if jop_imm32 <= self.maxJopImm:
                return True

        ###################################################
        # JMP/CALL [reg] with SIB
        # NOTE: Do no include pure [disp] instructions in SIB ([*] - none)
        elif (jop[0] in ["\x24","\x64","\xa4"] and not jop[1] in ["\x25","\x65","\xad","\xe5"]) or \
             (jop[0] in ["\x14","\x54","\x94"] and not jop[1] in ["\x25","\x65","\xad","\xe5"]):

             # Check for displacement
            if jop[0] in ["\x64","\x54"]:
                jop_imm8 = jop[2]
                jop_imm8 = unpack("b", jop_imm8)[0] # signed

                if jop_imm8 <= self.maxJopImm:
                    return True


            elif jop[0] in ["\xa4","\x94"]:
                jop_imm32 = jop[2:6]
                jop_imm32 = unpack("<i", jop_imm32)[0] # signed

                if jop_imm32 <= self.maxJopImm:
                    return True


            else:
                return True

        return False

    def is_sys_gadget(self, sys_op):
        """ syscall oriented gadget predicate """

        if sys_op[0] in [
            b"\xcd\x80",                         # int 0x80
            b"\x0f\x34",                         # sysenter
            b"\x0f\x05",                         # syscall
            b"\x65\xff\x15\x10\x00\x00\x00",     # call DWORD PTR gs:0x10
            b"\xcd\x80\xc3",                     # int 0x80 ; ret
            b"\x0f\x34\xc3",                     # sysenter ; ret
            b"\x0f\x05\xc3",                     # syscall ; ret
            b"\x65\xff\x15\x10\x00\x00\x00\xc3", # call DWORD PTR gs:0x10 ; ret
        ]:
            return True

        return False

    def search_job_gadgets(self, segment, jump_preamble = 0xFF):
        """ Search for JMP/J(.*) jump gadgets """

        # Nothing to do if the option is not set
        if not self.searchJop:
            return

        # Search all instances of JMP reg (FF /4) and CALL reg (FF /2)
        ea = segment.startEA
        while True:

            ea = idaapi.find_binary(ea + 1, segment.endEA, "%X" % jump_preamble, 16, idaapi.SEARCH_DOWN)
            if ea == idaapi.BADADDR: 
                break

            # Read possible ModR/M, SIB, and IMM8/IMM32 bytes
            jop = idc.GetManyBytes(ea + 1, self.maxRopSize)

            if self.is_job_gadget(jop):
                self.retns.append((ea))

    def search_sys_gadgets(self, segment, jump_preamble = 0xFF):
        """ Search for SYS jump gadgets """

        # Nothing to do if the option is not set
        if not self.searchSys:
            return

        # Search all instances of JMP reg (FF /4) and CALL reg (FF /2)
        ea = segment.startEA
        while True:

            ea = idaapi.find_binary(ea + 1, segment.endEA, "%X" % jump_preamble, 16, idaapi.SEARCH_DOWN)
            if ea == idaapi.BADADDR: 
                break

            # Read possible ModR/M, SIB, and IMM8/IMM32 bytes
            sys_op = idc.GetManyBytes(ea + 1, self.maxRopSize)

            if self.is_sys_gadget(jop):
                self.retns.append((ea))

    def search_rop_gadgets(self, segment, ret_preamble = 0xc3):
        """ Search for rop gadgets """

        # Nothing to do if the option is not set
        if not self.searchRop:
            return
            
        ea = segment.startEA
        while True:

            ea = idaapi.find_binary(ea + 1, segment.endEA, "%X" % ret_preamble, 16, idaapi.SEARCH_DOWN)
            if ea == idaapi.BADADDR: 
                break

            if ret_preamble in [ 0xc2, 0xca, 0xf2c2]:

                 # Read imm16 value and filter large values
                retn_imm16 = idc.Word(ea + 1)
                if retn_imm16 <= self.maxRetnImm:
                    self.retns.append((ea))
            else:
                self.retns.append( (ea))

            

    def search_gadgets(self):

        count_total = len(self.retns)
        count_notify = 0
        count_curr  = 0

        # BUG: A separate flag is used to track user canceling the search,
        #      because multiple calls to idaapi.wasBreak() do not properly
        #      detect cancellations.
        breakFlag = False

        # Show wait dialog
        if not self.debug: 
            idaapi.show_wait_box("Searching gadgets: 00 %")

        try :

        
            for ea_end in self.retns:

                # Flush the gadgets cache for each new retn pointer
                self.gadgets_cache = dict()

                # Flush memory cache for each new retn pointer
                self.dbg_mem_cache = None

                # CACHE: It is faster to read as much memory in one blob than to make incremental reads backwards.
                #        Try to read and cache self.maxRopOffset bytes back. In cases where it is not possible,
                #        then simply try to read the largest chunk.
                
                # NOTE: Read a bit extra to cover correct decoding of RETN, RETN imm16, CALL /2, and JMP /4 instructions.
                # Bug on end of segments : self.dbg_read_extra must be 0
                dbg_read_extra = self.dbg_read_extra
                seg_start, seg_end = idc.SegStart(ea_end), idc.SegEnd(ea_end)
                if ea_end + dbg_read_extra > seg_end:
                    dbg_read_extra = 0

                for i in range(self.maxRopOffset):
                    self.dbg_mem_cache = idc.GetManyBytes(ea_end - self.maxRopOffset + i, self.maxRopOffset - i + self.dbg_read_extra)
                    if self.dbg_mem_cache != None:
                        break

                # Error while reading memory (Ida sometimes does not want to read uninit data)
                if self.dbg_mem_cache == None:
                    for backward_size in range(self.maxRopOffset, 0, -1):
                        self.dbg_mem_cache = idc.GetManyBytes(ea_end - backward_size, backward_size)
                        if self.dbg_mem_cache != None:
                            break

                # Big problem ahead
                if self.dbg_mem_cache == None:
                    logging.error("[Ida Search Error] could not read bytes [0x%x, 0x%x]" % (ea_end - self.maxRopOffset + i, ea_end - self.maxRopOffset + i + self.maxRopOffset - i + self.dbg_read_extra))
                
                # Search all possible gadgets up to maxoffset bytes back
                # NOTE: Try all byte combinations to capture longer/more instructions
                #       even with bad bytes in the middle.
                for i in range(1, len(self.dbg_mem_cache) - self.dbg_read_extra):

                    ea = ea_end - i

                    # Try to build a gadget at the pointer
                    gadget = self.build_gadget(ea, ea_end)
                    
                    # Successfully built the gadget
                    if gadget:

                        # Filter gadgets with too many instruction
                        if gadget.size > self.maxRopSize: 
                            break

                        # Append newly built gadget
                        self.gadgets.append(gadget)
                        self.gadgets_cache[ea] = gadget

                        # Exceeded maximum number of gadgets
                        if self.maxRops and len(self.gadgets) >= self.maxRops:
                            breakFlag = True
                            print("[Ida Rop] Maximum number of gadgets exceeded.")
                            break
                    else:
                        self.gadgets_cache[ea] = None

                    if breakFlag or idaapi.wasBreak():
                        breakFlag = True
                        break


                # Canceled
                # NOTE: Only works when started from GUI not script.
                if breakFlag or idaapi.wasBreak():
                    breakFlag = True
                    print ("[IdaRopSearch] Canceled.")
                    break

                # Progress report
                if not self.debug and count_curr >= count_notify:

                    # NOTE: Need to use %%%% to escape both Python and IDA's format strings
                    percent_progression = count_curr*100/count_total
                    progression_str = """Searching gadgets: {progression:02d} %""".format(progression = percent_progression)
                    idaapi.replace_wait_box(progression_str) 

                    count_notify += 0.10 * count_total

                count_curr += 1            

            print ("[IdaRopSearch] Found %d gadgets." % len(self.gadgets))
        except:
            logging.error ("[IdaRopSearch] Exception raised while search for gadgets : %s." % sys.exc_info())
            pass

        finally:

            if not self.debug:
                idaapi.hide_wait_box()


    # Attempt to build a gadget at the provided start address
    # by verifying it properly terminates at the expected RETN.
    def build_gadget(self, ea, ea_end):

        instructions  = list()
        chg_registers = set()
        use_registers = set()
        operations    = set()
        pivot         = 0
        start_ea = ea

        # Process each instruction in the gadget
        while ea <= ea_end:

            ###################################################################
            # Gadget Level Cache:
            #
            # Locate a gadget (failed or built) starting at this address.
            # If one is located, then we don't need to process any further
            # instructions and just get necessary data from the cached
            # gadget to never have to process the same address twice.
            if ea in self.gadgets_cache:

                # Check if the gadget was build successfully
                gadget_cache = self.gadgets_cache[ea]
                
                # Build the reset of the gadget from cache
                if gadget_cache:    

                    for insn in gadget_cache.instructions:
                        instructions.append(insn)

                    #pivot += gadget_cache.pivot
                    opcodes = idc.GetManyBytes(start_ea, ea_end - start_ea + 1)
                    
                    gadget = Gadget(
                        address = start_ea,
                        ret_address = ea_end,
                        instructions = instructions,
                        opcodes = opcodes,
                        size = len(opcodes), 
                        #pivot = pivot
                        )

                    return gadget

                # Previous attempt to build gadget at this address failed
                else:
                    return None

            # Process new instruction
            else:

                # Instruction length
                # NOTE: decode_insn also sets global idaapi.cmd
                #       which contains insn_t structure
                insn_size = idaapi.decode_insn(ea)

                # Check successful decoding of the instruction
                if insn_size:

                    # Decoded instruction is too big to be a RETN or RETN imm16
                    if ea + insn_size > ea_end + self.dbg_read_extra:
                        return None

                    ###############################################################
                    # Instruction Level Cache
                    #
                    # Most instructions are repetitive so we can just cache
                    # unique byte combinations to avoid costly decoding more
                    # than once
                    
                    # Read instruction from memory cache
                    dbg_mem_offset = ea - (ea_end - (len(self.dbg_mem_cache) - self.dbg_read_extra) )
                    dbg_mem = self.dbg_mem_cache[dbg_mem_offset:dbg_mem_offset + insn_size]

                    # Create instruction cache if it doesn't already exist
                    if not dbg_mem in self.insn_cache:

                        ###########################################################
                        # Decode instruction
                        ###########################################################

                        # Get global insn_t structure describing the instruction
                        # NOTE: copy() is expensive, so we keep this single-threaded
                        insn = idaapi.cmd                  

                        #######################################################
                        # Decode and Cache instruction characteristics
                        self.insn_cache[dbg_mem] = self.decode_instruction(insn, ea, ea_end)

                    ##################################################################
                    # Retrieve cached instruction and apply it to the gadget    

                    # Check that cached instruction contains valid data
                    if self.insn_cache[dbg_mem]:

                        # Retrieve basic instruction characteristics
                        insn_mnem = self.insn_cache[dbg_mem]["insn_mnem"]
                        insn_disas = self.insn_cache[dbg_mem]["insn_disas"]
                        instructions.append(insn_disas)

                        #######################################################
                        # Expected ending instruction of the gadget
                        if ea == ea_end:
                            opcodes = idc.GetManyBytes(start_ea, ea_end - start_ea + 1)
                            
                            gadget = Gadget(
                                address = start_ea,
                                ret_address = ea_end,
                                instructions = instructions,
                                opcodes = opcodes,
                                size = len(opcodes), 
                                #pivot = pivot
                                )
                            return gadget

                        #######################################################
                        # Filter out of place ROP/JOP/COP terminators
                        # NOTE: retn/jmp/call are allowed, but only in the last position

                        # Unexpected return instruction
                        elif insn_mnem == "retn":
                            return None

                        # Unexpected call/jmp instruction
                        elif insn_mnem in ["jmp","call"]:
                            return None

                        #######################################################
                        # Add instruction instruction characteristics to the gadget
                        else:

                            for reg in self.insn_cache[dbg_mem]["insn_chg_registers"]:
                                chg_registers.add(reg)

                            for reg in self.insn_cache[dbg_mem]["insn_use_registers"]:
                                use_registers.add(reg)

                            for op in self.insn_cache[dbg_mem]["insn_operations"]:
                                operations.add(op)

                            pivot += self.insn_cache[dbg_mem]["insn_pivot"]

                    # Previous attempt to decode the instruction invalidated the gadget
                    else:                        
                        return None
                        
                    ###############################################################
                    # Next instruction
                    # NOTE: This is outside cache
                    ea += insn_size

                ###################################################################
                # Failed decoding of the instruction
                # NOTE: Gadgets may have bad instructions in the middle which
                #       can be tolerated as long as we can find a useful instruction
                #       further out.
                else:

                    # HACK: IDA does not disassemble "\x00\x00" unless you enable
                    #       "Disassemble zero opcode instructions" in Processor Options.
                    #       Since this option is normally disabled, I will attempt
                    #       to get this instruction manually.

                    # Read two bytes from memory cache at current instruction candidate
                    dbg_mem_offset = ea - (ea_end - self.maxRopOffset)
                    dbg_mem = self.dbg_mem_cache[dbg_mem_offset:dbg_mem_offset + 2]

                    # Compare to two zero bytes
                    if dbg_mem[:2] == "\x00\x00":

                        if self.sploiter.addr64:
                            instructions.append("add [rax],al")
                        else:
                            instructions.append("add [eax],al")

                        use_registers.add("al")
                        operations.add("reg-to-mem")

                        ea += 2

                    # "MOV Sreg, r/m16" instructions will result in illegal instruction exception: c000001d
                    # or the memory couldn't be read exception: c0000005 which we don't want in our gadgets.
                    elif len(dbg_mem) and dbg_mem[0] == "\x8E":
                        return None

                    # Record a "bad byte" if allowed
                    elif dbg_mem and not self.ropNoBadBytes:
                        byte = dbg_mem[0]

                        instructions.append("db %sh" % binascii.hexlify(byte))

                        ea += 1

                    # Invalidate the gadget
                    else:
                        return None

        # Failed to build a gadget, because RETN instruction was not found
        else:    
            return None

    ###############################################################
    # Decode instruction

    def decode_instruction(self, insn, ea, ea_end):

        # Instruction specific characteristics
        insn_chg_registers = set()
        insn_use_registers = set()
        insn_operations = set()
        insn_pivot = 0

        # Instruction feature
        #
        # instruc_t.feature
        #
        # CF_STOP = 0x00001 #  Instruction doesn't pass execution to the next instruction
        # CF_CALL = 0x00002 #  CALL instruction (should make a procedure here)
        # CF_CHG1 = 0x00004 #  The instruction modifies the first operand
        # CF_CHG2 = 0x00008 #  The instruction modifies the second operand
        # CF_CHG3 = 0x00010 #  The instruction modifies the third operand
        # CF_CHG4 = 0x00020 #  The instruction modifies 4 operand
        # CF_CHG5 = 0x00040 #  The instruction modifies 5 operand
        # CF_CHG6 = 0x00080 #  The instruction modifies 6 operand
        # CF_USE1 = 0x00100 #  The instruction uses value of the first operand
        # CF_USE2 = 0x00200 #  The instruction uses value of the second operand
        # CF_USE3 = 0x00400 #  The instruction uses value of the third operand
        # CF_USE4 = 0x00800 #  The instruction uses value of the 4 operand
        # CF_USE5 = 0x01000 #  The instruction uses value of the 5 operand
        # CF_USE6 = 0x02000 #  The instruction uses value of the 6 operand
        # CF_JUMP = 0x04000 #  The instruction passes execution using indirect jump or call (thus needs additional analysis)
        # CF_SHFT = 0x08000 #  Bit-shift instruction (shl,shr...)
        # CF_HLL  = 0x10000 #  Instruction may be present in a high level language function.
        insn_feature = insn.get_canon_feature()

        # Instruction mnemonic name
        insn_mnem = insn.get_canon_mnem()

        #if insn_mnem in self.mnems: self.mnems[insn_mnem] += 1
        #else:                       self.mnems[insn_mnem]  = 1

        # Get instruction operand types
        #
        # op_t.type
        #                    Description                          Data field
        # o_void     =  0 #  No Operand                           ----------
        # o_reg      =  1 #  General Register (al,ax,es,ds...)    reg
        # o_mem      =  2 #  Direct Memory Reference  (DATA)      addr
        # o_phrase   =  3 #  Memory Ref [Base Reg + Index Reg]    phrase
        # o_displ    =  4 #  Memory Reg [Base Reg + Index Reg + Displacement] phrase+addr
        # o_imm      =  5 #  Immediate Value                      value
        # o_far      =  6 #  Immediate Far Address  (CODE)        addr
        # o_near     =  7 #  Immediate Near Address (CODE)        addr
        insn_op1 = insn.Operands[0].type
        insn_op2 = insn.Operands[1].type

        ###############################################################
        # Filter gadget
        ###############################################################

        # Do not filter ROP, JOP, COP, always decode them
        # NOTE: A separate check must be done to check if they are out of place.
        if not insn_mnem in ["retn","jmp","call"]:

            # Filter gadgets with instructions that don't forward execution to the next address
            if insn_feature & idaapi.CF_STOP:
                return None

            # Filter gadgets with instructions in a bad list
            elif insn_mnem in self.ropBadMnems:
                return None

            # Filter gadgets with jump instructions
            # Note: conditional jumps may still be useful if we can
            #       set flags prior to calling them.
            elif not self.ropAllowJcc and insn_mnem[0] == "j":
                return None

        ###############################################################
        # Get disassembly
        ###############################################################
        # NOTE: GENDSM_FORCE_CODE ensures correct decoding
        #       of split instructions.
        insn_disas = idc.GetDisasmEx(ea, idaapi.GENDSM_FORCE_CODE)                
        insn_disas = insn_disas.partition(';')[0]       # Remove comments from disassembly                       
        insn_disas = ' '.join(insn_disas.split())       # Remove extraneous space from disassembly

        ###############################################################
        # Analyze instruction
        ###############################################################

        # Standalone instruction
        if insn_op1 == idaapi.o_void:

            # TODO: Determine and test how these instructions affect the stack
            #       in 32-bit and 64-bit modes.
            if insn_mnem in ["pusha","pushad","popa","popad","pushf","pushfd","pushfq","popf","popfd","popfq"]:
                insn_operations.add("stack")

                if insn_mnem in ["popa","popad"]:
                    insn_pivot += 7*4
                elif insn_mnem in ["pusha","pushad"]:
                    insn_pivot -= 8*4
                elif insn_mnem in ["popf","popfd"]:
                    insn_pivot += 4
                elif insn_mnem in ["pushf","pushfd"]:
                    insn_pivot -= 4
                elif insn_mnem == "popfq":  # TODO: Needs testing
                    insn_pivot += 8
                elif insn_mnem == "pushfq": # TODO: Needs testing
                    insn_pivot -= 8

        # Single operand instruction
        elif insn_op2 == idaapi.o_void:

            # Single operand register
            if insn_op1 == idaapi.o_reg:
                insn_operations.add("one-reg")

                if insn_feature & idaapi.CF_CHG1:
                    reg_name = self.get_o_reg_name(insn, 0)
                    insn_chg_registers.add(reg_name)

                    # Check for stack operation
                    if reg_name[1:] == "sp":
                        insn_operations.add("stack")

                        if insn_mnem == "inc":
                            insn_pivot += 1

                        elif insn_mnem == "dec":
                            insn_pivot -= 1

                elif insn_feature & idaapi.CF_USE1:
                    reg_name = self.get_o_reg_name(insn, 0)
                    insn_use_registers.add(reg_name)

            # Single operand immediate
            elif insn_op1 == idaapi.o_imm:
                insn_operations.add("one-imm")

            # Single operand reference
            # TODO: determine the [reg + ...] value if present
            elif insn_op1 == idaapi.o_phrase or insn_op1 == idaapi.o_displ:
                insn_operations.add("one-mem")

            # PUSH/POP mnemonic with a any operand type
            if insn_mnem in ["push","pop"]:
                insn_operations.add("stack")

                # Adjust pivot based on operand size (32bit vs 64bit)
                if insn_mnem == "pop":
                    if   insn.Operands[0].dtyp == idaapi.dt_dword: insn_pivot += 4
                    elif insn.Operands[0].dtyp == idaapi.dt_qword: insn_pivot += 8
                elif insn_mnem == "push":                   
                    if   insn.Operands[0].dtyp == idaapi.dt_dword: insn_pivot -= 4
                    elif insn.Operands[0].dtyp == idaapi.dt_qword: insn_pivot -= 8

            # Check for arithmetic operation:
            if insn_mnem in self.insn_arithmetic_ops:
                insn_operations.add("math")

            # Check for bit-wise operations:
            if insn_mnem in self.insn_bit_ops:
                insn_operations.add("bit")

        # Two operand instruction
        else:

            # Check for arithmetic operations
            if insn_mnem in self.insn_arithmetic_ops:
                insn_operations.add("math")

            # Check for bit-wise operations
            if insn_mnem in self.insn_bit_ops:
                insn_operations.add("bit")

            # Two operand instruction with the first operand a register
            if insn_op1 == idaapi.o_reg:

                reg_name = self.get_o_reg_name(insn, 0)

                if insn_feature & idaapi.CF_CHG1:
                    insn_chg_registers.add(reg_name)

                    # Check for stack operation
                    if reg_name[1:] == "sp":
                        insn_operations.add("stack")

                        # Determine stack pivot distance
                        if insn_op2 == idaapi.o_imm:

                            # NOTE: adb and sbb may also be useful, but let the user
                            #       determine their use by locating the operations "stack"
                            if insn_mnem == "add":
                                insn_pivot += insn.Operands[1].value

                            elif insn_mnem == "sub":
                                insn_pivot -= insn.Operands[1].value

                    # Check for operations
                    if insn_op2 == idaapi.o_reg:
                        insn_operations.add("reg-to-reg")
                    elif insn_op2 == idaapi.o_imm:
                        insn_operations.add("imm-to-reg")

                    # TODO: determine the [reg + ...] value if present
                    elif insn_op2 == idaapi.o_phrase or insn_op2 == idaapi.o_displ:
                        insn_operations.add("mem-to-reg")

                if insn_feature & idaapi.CF_USE1:
                    insn_use_registers.add(reg_name)


            # Two operand instruction with the second operand a register
            if insn_op2 == idaapi.o_reg:

                reg_name = self.get_o_reg_name(insn, 1)

                if insn_feature & idaapi.CF_CHG2:
                    insn_chg_registers.add(reg_name)

                    # Check for stack operation
                    if reg_name[1:] == "sp":
                        insn_operations.add("stack")

                if insn_feature & idaapi.CF_USE2:
                    insn_use_registers.add(reg_name)

                # Check for operations
                # TODO: determine the [reg + ...] value if present
                if insn_op1 == idaapi.o_phrase or insn_op1 == idaapi.o_displ:
                    insn_operations.add("reg-to-mem")

        # Build instruction dictionary
        insn = dict()
        insn["insn_mnem"] = insn_mnem
        insn["insn_disas"] = insn_disas
        insn["insn_operations"] = insn_operations
        insn["insn_chg_registers"] = insn_chg_registers
        insn["insn_use_registers"] = insn_use_registers
        insn["insn_pivot"] = insn_pivot

        return insn


class IdaRopEngine():
    """ Ida ROP Engine class for process all kinds of data """

    def __init__(self):
        self.rop  = None

        if not idaapi.ph.id == idaapi.PLFM_386:
            logging.error ("[IdaRop] Only Intel 80x86 processors are supported.")
            sys.exit(1)

        # Check if processor supports 64-bit addressing
        if idaapi.ph.flag & idaapi.PR_USE64:
            self.addr64 = True
            self.addr_format = "%016X"
            self.pack_format_be = ">Q"
            self.pack_format_le = "<Q"
        else:
            self.addr64 = False
            self.addr_format = "%08X"
            self.pack_format_be = ">I"
            self.pack_format_le = "<I"

    def list_segments(self):
        """ Return the list of segments in the current binary and their characteristics """
        
        self.segments = list()
        self.segments_idx = list()


        for n in xrange(get_segm_qty()):
            seg = getnseg(n)

            if not seg: 
                continue

            # For some linux binaries
            # Ida does not recognize the segment
            # permissions (usually for plt)
            if seg.perm == 0:
                continue

            segentry = SegmentEntry(
                name = idaapi.get_segm_name(seg),
                start = seg.startEA,
                end = seg.endEA,
                size = seg.size(),
                r = (seg.perm & idaapi.SEGPERM_READ) >> 2,
                w = (seg.perm & idaapi.SEGPERM_WRITE) >> 1,
                x = (seg.perm & idaapi.SEGPERM_EXEC),
                segclass = idaapi.get_segm_class(seg)
            )

            self.segments.append(segentry) 
            self.segments_idx.append(n)

        return self.segments

    def clear_rop_list(self):
        """ Clear previous rop search results """
        self.rop.gadgets = list()

    def process_rop(self, form,  select_list = None):
        """ Look for rop gadgets using user-input search options """
        
        # Clear previous results
        self.clear_rop_list()
        
        # Get selected segments
        self.rop.segments = [self.segments_idx[i] for i in select_list]

        if len(self.rop.segments) > 0:

            # Filter bad characters
            buf                    = form.strBadChars.value
            buf = buf.replace(' ','')         # remove spaces
            buf = buf.replace('\\x','')       # remove '\x' prefixes
            buf = buf.replace('0x','')        # remove '0x' prefixes
            try:
                buf = binascii.unhexlify(buf) # convert to bytes
                self.ptrBadChars   = buf
            except Exception as e:
                idaapi.warning("Invalid input: %s" % e)
                self.ptrBadChars   = ""

            # Ascii_to_Unicode_transformation table
            # BUG: DropdownControl does not work on IDA 6.5
            self.unicodeTable      = form.radUnicode.value

            # ROP instruction filters
            self.rop.ropBadMnems   = [mnem.strip().lower() for mnem in form.strBadMnems.value.split(',')]
            self.rop.ropAllowJcc   = form.cRopAllowJcc.checked
            self.rop.ropNoBadBytes = form.cRopNoBadBytes.checked

            # Get ROP engine settings
            self.rop.maxRopSize    = form.intMaxRopSize.value
            self.rop.maxRopOffset  = form.intMaxRopOffset.value
            self.rop.maxRops       = form.intMaxRops.value
            self.rop.maxRetnImm    = form.intMaxRetnImm.value

            # Gadget search values
            self.rop.searchRop     = form.cRopSearch.checked
            self.rop.searchJop     = form.cJopSearch.checked
            self.rop.searchSys     = form.cSysSearch.checked 

            # Search for returns and ROP gadgets
            self.rop.search_retns()
            self.rop.search_gadgets()

            return True

        else:
            idaapi.warning("No segments selected.")
            return False

       


import zlib
import json
import logging

import idaapi

BLOB_SIZE = 1024
OUR_NETNODE = "$ com.williballenthin"
INT_KEYS_TAG = 'M'
STR_KEYS_TAG = 'N'
STR_TO_INT_MAP_TAG = 'O'
INT_TO_INT_MAP_TAG = 'P'
logger = logging.getLogger(__name__)


class NetnodeCorruptError(RuntimeError):
    pass


class Netnode(object):
    """
    A netnode is a way to persistently store data in an IDB database.
    The underlying interface is a bit weird, so you should read the IDA
      documentation on the subject. Some places to start:

      - https://www.hex-rays.com/products/ida/support/sdkdoc/netnode_8hpp.html
      - The IDA Pro Book, version 2
 
    Conceptually, this netnode class represents is a key-value store
      uniquely identified by a namespace.

    This class abstracts over some of the peculiarities of the low-level
      netnode API. Notably, it supports indexing data by strings or
      numbers, and allows values to be larger than 1024 bytes in length.
    
    This class supports keys that are numbers or strings. 
    Values must be JSON-encodable. They can not be None.
   
    Implementation:
     (You don't have to worry about this section if you just want to
        use the library. Its here for potential contributors.)

      The major limitation of the underlying netnode API is the fixed
        maximum length of a value. Values must not be larger than 1024
        bytes. Otherwise, you must use the `blob` API. We do that for you.
    
      The first enhancement is transparently zlib-encoding all values.

      To support arbitrarily sized values with keys of either int or str types,
        we store the values in different places:
    
        - integer keys with small values: stored in default supval table
        - integer keys with large values: the data is stored in the blob
           table named 'M' using an internal key. The link from the given key
           to the internal key is stored in the supval table named 'P'.
        - string keys with small values: stored in default hashval table
        - string keys with large values: the data is stored in the blob
           table named 'N' using an integer key. The link from string key
           to int key is stored in the supval table named 'O'.
    """
    def __init__(self, netnode_name=OUR_NETNODE):
        self._netnode_name = netnode_name
        #self._n = idaapi.netnode(netnode_name, namelen=0, do_create=True)
        self._n = idaapi.netnode(netnode_name, 0, True)

    @staticmethod
    def _decompress(data):
        return zlib.decompress(data)

    @staticmethod
    def _compress(data):
        return zlib.compress(data)

    @staticmethod
    def _encode(data):
        return json.dumps(data)

    @staticmethod
    def _decode(data):
        return json.loads(data)

    def _intdel(self, key):
        assert isinstance(key, (int, long))

        did_del = False
        storekey = self._n.supval(key, INT_TO_INT_MAP_TAG)
        if storekey is not None:
            storekey = int(storekey)
            self._n.delblob(storekey, INT_KEYS_TAG)
            self._n.supdel(key)
            did_del = True
        if self._n.supval(key) is not None:
            self._n.supdel(key)
            did_del = True

        if not did_del:
            raise KeyError("'{}' not found".format(key))

    def _get_next_slot(self, tag):
        '''
        get the first unused supval table key, or 0 if the
         table is empty.
        useful for filling the supval table sequentially.
        '''
        slot = self._n.suplast(tag)
        if slot is None or slot == idaapi.BADNODE:
            return 0
        else:
            return slot + 1

    def _intset(self, key, value):
        assert isinstance(key, (int, long))
        assert value is not None

        try:
            self._intdel(key)
        except KeyError:
            pass

        if len(value) > BLOB_SIZE:
            storekey = self._get_next_slot(INT_KEYS_TAG)
            self._n.setblob(value, storekey, INT_KEYS_TAG)
            self._n.supset(key, str(storekey), INT_TO_INT_MAP_TAG)
        else:
            self._n.supset(key, value)

    def _intget(self, key):
        assert isinstance(key, (int, long))

        storekey = self._n.supval(key, INT_TO_INT_MAP_TAG)
        if storekey is not None:
            storekey = int(storekey)
            v = self._n.getblob(storekey, INT_KEYS_TAG)
            if v is None:
                raise NetnodeCorruptError()
            return v

        v = self._n.supval(key)
        if v is not None:
            return v

        raise KeyError("'{}' not found".format(key))

    def _strdel(self, key):
        assert isinstance(key, (basestring))

        did_del = False
        storekey = self._n.hashval(key, STR_TO_INT_MAP_TAG)
        if storekey is not None:
            storekey = int(storekey)
            self._n.delblob(storekey, STR_KEYS_TAG)
            self._n.hashdel(key)
            did_del = True
        if self._n.hashval(key):
            self._n.hashdel(key)
            did_del = True

        if not did_del:
            raise KeyError("'{}' not found".format(key))

    def _strset(self, key, value):
        assert isinstance(key, (basestring))
        assert value is not None

        try:
            self._strdel(key)
        except KeyError:
            pass

        if len(value) > BLOB_SIZE:
            storekey = self._get_next_slot(STR_KEYS_TAG)
            self._n.setblob(value, storekey, STR_KEYS_TAG)
            self._n.hashset(key, str(storekey), STR_TO_INT_MAP_TAG)
        else:
            self._n.hashset(key, value)

    def _strget(self, key):
        assert isinstance(key, (basestring))

        storekey = self._n.hashval(key, STR_TO_INT_MAP_TAG)
        if storekey is not None:
            storekey = int(storekey)
            v = self._n.getblob(storekey, STR_KEYS_TAG)
            if v is None:
                raise NetnodeCorruptError()
            return v

        v = self._n.hashval(key)
        if v is not None:
            return v

        raise KeyError("'{}' not found".format(key))

    def __getitem__(self, key):
        if isinstance(key, basestring):
            v = self._strget(key)
        elif isinstance(key, (int, long)):
            v = self._intget(key)
        else:
            raise TypeError("cannot use {} as key".format(type(key)))

        return self._decode(self._decompress(v))

    def __setitem__(self, key, value):
        '''
        does not support setting a value to None.
        value must be json-serializable.
        key must be a string or integer.
        '''
        assert value is not None

        v = self._compress(self._encode(value))
        if isinstance(key, basestring):
            self._strset(key, v)
        elif isinstance(key, (int, long)):
            self._intset(key, v)
        else:
            raise TypeError("cannot use {} as key".format(type(key)))

    def __delitem__(self, key):
        if isinstance(key, basestring):
            self._strdel(key)
        elif isinstance(key, (int, long)):
            self._intdel(key)
        else:
            raise TypeError("cannot use {} as key".format(type(key)))

    def get(self, key, default=None):
        try:
            return self[key]
        except KeyError:
            return default

    def __contains__(self, key):
        try:
            if self[key] is not None:
                return True
            return False
        except KeyError:
            return False

    def iterkeys(self):
        # integer keys for all small values
        i = self._n.sup1st()
        while i != idaapi.BADNODE:
            yield i
            i = self._n.supnxt(i)

        # integer keys for all big values
        i = self._n.sup1st(INT_TO_INT_MAP_TAG)
        while i != idaapi.BADNODE:
            yield i
            i = self._n.supnxt(i, INT_TO_INT_MAP_TAG)

        # string keys for all small values
        i = self._n.hash1st()
        while i != idaapi.BADNODE and i is not None:
            yield i
            i = self._n.hashnxt(i)

        # string keys for all big values
        i = self._n.hash1st(STR_TO_INT_MAP_TAG)
        while i != idaapi.BADNODE and i is not None:
            yield i
            i = self._n.hashnxt(i, STR_TO_INT_MAP_TAG)

    def keys(self):
        return [k for k in self.iterkeys()]

    def itervalues(self):
        for k in self.iterkeys():
            yield self[k]

    def values(self):
        return [v for v in self.itervalues()]

    def iteritems(self):
        for k in self.iterkeys():
            yield k, self[k]

    def items(self):
        return [(k, v) for k, v in self.iteritems()]

    def kill(self):
        self._n.kill()
        self._n = idaapi.netnode(self._netnode_name, 0, True)




# IDA libraries
import idaapi
from idaapi import plugin_t

from idarop import IDAROP_VERSION, IDAROP_DESCRIPTION
from idarop.ui import IdaRopManager

class idarop_t(plugin_t):

    flags = idaapi.PLUGIN_UNL
    comment = IDAROP_DESCRIPTION
    help = IDAROP_DESCRIPTION
    wanted_name = "IDA ROP"
    wanted_hotkey = ""

    def init(self):
        """ On script initalisation : load previous rop results and init menu items """

        # Only Intel x86/x86-64 are supported
        if idaapi.ph_get_id() == idaapi.PLFM_386:

            global idarop_manager

            # Check if already initialized
            if not 'idarop_manager' in globals():

                idarop_manager = IdaRopManager()
                if idarop_manager.add_menu_items():
                    print("[IdaRop] Failed to initialize IDA Sploiter.")
                    idarop_manager.del_menu_items()
                    del idarop_manager
                    return idaapi.PLUGIN_SKIP
                else:
                    try:
                        idarop_manager.load_internal_db()
                    except Exception as e:
                        pass

                    print("[IdaRop] IDA ROP View  v%s initialized " % IDAROP_VERSION)

            return idaapi.PLUGIN_KEEP
        else:
            return idaapi.PLUGIN_SKIP

    def run(self, arg):
        pass

    def term(self):
        """ On IDA's close event, export the Rop gadget list in a default csv file"""
        idaapi.show_wait_box("Saving gadgets ...")
        try:
            idarop_manager.save_internal_db()
        except Exception as e:
            pass
        idaapi.hide_wait_box()

def PLUGIN_ENTRY():
    return idarop_t()

###############################################################################
# Script / Testing
###############################################################################

def idarop_main():
    global idarop_manager

    if 'idarop_manager' in globals():
        idarop_manager.del_menu_items()
        del idarop_manager

    idarop_manager = IdaRopManager()
    idarop_manager.add_menu_items()

    idarop = idarop_manager.idarop

if __name__ == '__main__':
    #idarop_main()
    pass


import os
import random
import string
import logging
import contextlib

import idaapi
import pytest

import netnode


TEST_NAMESPACE = '$ some.namespace'


def get_random_data(N):
    return ''.join(random.choice(string.ascii_uppercase + string.digits) for _ in range(N))


@contextlib.contextmanager
def killing_netnode(namespace):
    '''
    wraps a netnode in a contextmanager that will
     eventually destroy its contents.
    probably only useful for testing when a clean state is req'd.
    '''
    n = netnode.Netnode(namespace)
    try:
        yield n
    finally:
        n.kill()


def test_basic_features():
    '''
    demonstrate the basic netnode API (like a dict)
    '''
    with killing_netnode(TEST_NAMESPACE) as n:
        # there is nothing in the netnode to begin with
        assert(False == (1 in n))

        # when we add one key, there is one thing in it
        n[1] = 'hello'
        assert(True == (1 in n))
        assert(n[1] == 'hello')
        # but nothing else
        assert(False == ('2' in n))

        # then when we add a second thing, its also there
        n['2'] = 'world'
        assert(True == ('2' in n))
        assert(len(n.keys()) == 2)
        assert(n.keys()[0] == 1)
        assert(n.keys()[1] == '2')
        assert(len(n.values()) == 2)
        assert(n.values()[0] == 'hello')
        assert(n.values()[1] == 'world')
        assert(len(n.items()) == 2)

        # and when we delete the first item, only it is deleted
        del n[1]
        assert(False == (1 in n))

        # and finally everything is gone
        del n['2']


def test_large_data():
    '''
    demonstrate that netnodes support large data values.
    '''
    with killing_netnode(TEST_NAMESPACE) as n:
        random_data = get_random_data(1024 * 8)
        n[3] = random_data
        assert(n[3] == random_data)


def test_hash_ordering():
    ''' 
    the following demonstrates that 'hashes' are iterated alphabetically.
    this is an IDAPython implementation feature.
    '''

    with killing_netnode(TEST_NAMESPACE) as n:
        m = n._n

        def hashiter(m):
            i = m.hash1st()
            while i != idaapi.BADNODE and i is not None:
                yield i
                i = m.hashnxt(i)

        def get_hash_order(hiter):
            return [k for k in hiter]

        m.hashset('a', 'a')
        assert get_hash_order(hashiter(m)) == ['a']

        m.hashset('c', 'c')
        assert get_hash_order(hashiter(m)) == ['a', 'c']

        m.hashset('b', 'b')
        assert get_hash_order(hashiter(m)) == ['a', 'b', 'c']


def test_iterkeys():
    LARGE_VALUE = get_random_data(16 * 1024)
    LARGE_VALUE2 = get_random_data(16 * 1024)
    import zlib
    assert(zlib.compress(LARGE_VALUE) > 1024)
    assert(zlib.compress(LARGE_VALUE2) > 1024)

    assert LARGE_VALUE != LARGE_VALUE2

    with killing_netnode(TEST_NAMESPACE) as n:
        n[1] = LARGE_VALUE
        assert set(n.keys()) == set([1])

        n[2] = LARGE_VALUE2
        assert set(n.keys()) == set([1, 2])

        assert n[1] != n[2]

    with killing_netnode(TEST_NAMESPACE) as n:
        n['one'] = LARGE_VALUE
        assert set(n.keys()) == set(['one'])

        n['two'] = LARGE_VALUE2
        assert set(n.keys()) == set(['one', 'two'])

        assert n['one'] != n['two']
        
    with killing_netnode(TEST_NAMESPACE) as n:
        n[1] = LARGE_VALUE
        assert set(n.keys()) == set([1])

        n[2] = LARGE_VALUE
        assert set(n.keys()) == set([1, 2])

        n['one'] = LARGE_VALUE
        assert set(n.keys()) == set([1, 2, 'one'])

        n['two'] = LARGE_VALUE
        assert set(n.keys()) == set([1, 2, 'one', 'two'])

        n[3] = "A"
        assert set(n.keys()) == set([1, 2, 'one', 'two', 3])
 
        n['three'] = "A"
        assert set(n.keys()) == set([1, 2, 'one', 'two', 3, 'three'])
 

def main():
    logging.basicConfig(level=logging.DEBUG)

    # cleanup any existing data
    netnode.Netnode(TEST_NAMESPACE).kill()

    pytest.main(['--capture=sys', os.path.dirname(__file__)])


if __name__ == '__main__':
    main()



import os
from io import open
from setuptools import setup, find_packages

from __ida_setup__ import IdaPluginInstallCommand
from idarop import IDAROP_VERSION, IDAROP_DESCRIPTION


# read the contents of README file
package_directory = os.path.abspath(os.path.dirname(__file__))
readme_path = os.path.join(package_directory, 'README.md')
with open(readme_path, "r", encoding = 'utf-8') as f:
    long_description = f.read()


setup(
    name = 'idarop',
    version = IDAROP_VERSION,
    description = IDAROP_DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    author = "lucasg",
    author_email = "lucas.georges@outlook.com",
    url = "https://github.com/lucasg/idarop",
    
    install_requires = [
    ],

    packages = find_packages(),
    py_modules = ['__ida_setup__', 'plugins/idarop_plugin_t'],

    # Declare your ida plugins here
    package_data = {
        'ida_plugins': ['plugins/idarop_plugin_t.py'],
    },
    
    include_package_data =True,
    
    # monkey patch install script for IDA plugin custom install
    cmdclass={'install': IdaPluginInstallCommand}   
)


import os
import sys
import getpass
import distutils
from distutils.core import setup
from setuptools.command.install import install


if (sys.version_info > (3, 0)):
    raise ImportError("Idapython runs in a Python 2.7 interpreter, please execute this install setup with it.")

def ida_install_dir_windows(version, *args):
    IDA_INSTALL_DIR_WINDOWS = {
        '6.8' : os.path.join(os.environ.get("ProgramFiles(x86)", "KeyError"), "IDA 6.8", "plugins"),
        '6.9' : os.path.join(os.environ.get("ProgramFiles(x86)", "KeyError"), "IDA 6.9", "plugins"),
        '7.0' : os.path.join(os.environ.get("ProgramW6432", "KeyError"), "IDA 7.0", "plugins"),
        '7.1' : os.path.join(os.environ.get("ProgramW6432", "KeyError"), "IDA 7.1", "plugins"),
    }

    return IDA_INSTALL_DIR_WINDOWS[version]

def ida_install_dir_macos(version, *args):
    IDA_INSTALL_DIR_MACOS = {
        '6.8' : os.path.join("/Applications", "IDA Pro 6.8", "idaq.app/Contents/MacOS/plugins"),
        '6.9' : os.path.join("/Applications", "IDA Pro 6.9", "idaq.app/Contents/MacOS/plugins"),
        '7.0' : os.path.join("/Applications", "IDA Pro 7.0", "idaq.app/Contents/MacOS/plugins"),
        '7.1' : os.path.join("/Applications", "IDA Pro 7.1", "idaq.app/Contents/MacOS/plugins"),
    }

    return IDA_INSTALL_DIR_MACOS[version]

def ida_install_dir_linux(version, is_user, *args):
    IDA_INSTALL_DIR_LINUX_USER = {
        '6.8' : os.path.join("/home", getpass.getuser() ,"IDA 6.8", "plugins"),
        '6.9' : os.path.join("/home", getpass.getuser() ,"IDA 6.9", "plugins"),
        '7.0' : os.path.join("/home", getpass.getuser() ,"IDA 7.0", "plugins"),
        '7.1' : os.path.join("/home", getpass.getuser() ,"IDA 7.1", "plugins"),
    }

    IDA_INSTALL_DIR_LINUX_SYSTEM = {
        '6.8' : os.path.join("/opt", "IDA 6.8", "plugins"),
        '6.9' : os.path.join("/opt", "IDA 6.9", "plugins"),
        '7.0' : os.path.join("/opt", "IDA 7.0", "plugins"),
        '7.1' : os.path.join("/opt", "IDA 7.1", "plugins"),
    }

    if is_user:
        return IDA_INSTALL_DIR_LINUX_USER[version]
    else:
        return IDA_INSTALL_DIR_LINUX_SYSTEM[version]

IDA_SUPPORTED_VERSIONS = ('6.8','6.9','7.0','7.1')

IDA_INSTALL_DIRS = {
    
    # On Windows, the folder is at C:\Program Files (x86)\IDA %d\plugins
    'win32' : ida_install_dir_windows,

    'cygwin': ida_install_dir_windows,

    # On MacOS, the folder is at /Applications/IDA\ Pro\ %d/idaq.app/Contents/MacOS/plugins
    'darwin' : ida_install_dir_macos,

    # On Linux, the folder may be at /opt/IDA/plugins/
    'linux2' : ida_install_dir_linux,

    # Python 3 
    'linux' : ida_install_dir_linux,
}       

class IdaPluginInstallCommand(install):
    description = "install the current plugin in IDA plugin folder."
    user_options = install.user_options + [
        ('ida', None, 'force custom ida install script.'),
        ('ida-version=', None, 'specify ida version.'),
        ('ida-install-deps', None, 'install ida plugin dependencies.'),
    ]

    def initialize_options(self):
        install.initialize_options(self)
        self.ida = False # explicitely tell setuptools to use the ida setup script
        self.ida_version = None # locate default ida version
        self.ida_install_deps = False # Install plugin deps

    def finalize_options(self):
        
        # Search for a supported version installed
        if self.ida_version == None:

            for ida_version in IDA_SUPPORTED_VERSIONS:
                ida_install_dir = IDA_INSTALL_DIRS[sys.platform](ida_version, self.user)

                if os.path.exists(ida_install_dir):
                    self.ida_version = ida_version
                    self.announce("[IDA PLUGIN INSTALL] No ida version provided, using default version : %s" % self.ida, level=distutils.log.ERROR)
                    break

            

        assert self.ida_version in IDA_SUPPORTED_VERSIONS, 'Supported IDA on this platform : %s' % IDA_SUPPORTED_VERSIONS
        install.finalize_options(self)

    def install_dependencies(self, dist, install_dir):
        # type:  (distutils.core.install, setuptools.dist.Distribution, str) -> void
        """ Recursively install dependency using pip (for those on pipy) """

        if not len(dist.install_requires):
            return


        # inner import in order to prevent build breakage 
        # on old Python2 installs with no pip package unless 
        # there is actually a need for it.
        import pip

        for dependency in dist.install_requires:
            self.announce("[IDA PLUGIN INSTALL] installing dependency %s -> %s" % (dependency, install_dir), level=distutils.log.INFO)
            
            if not self.dry_run:
                pip.main(['install', '-t', install_dir, "--ignore-installed" ,  dependency])

    def install_packages(self, dist, install_dir):
        # type:  (distutils.core.install, setuptools.dist.Distribution, str) -> void
        """ Install python packages """

        for package in dist.packages:
            self.announce("[IDA PLUGIN INSTALL] copy package %s -> %s" % (package, install_dir), level=distutils.log.INFO)

            if not self.dry_run:
                self.copy_tree(package, os.path.join(install_dir, package))

    def install_plugins(self, dist, install_dir):
        # type:  (distutils.core.install, setuptools.dist.Distribution, str) -> void
        """ Install ida plugins entry points """

        ida_plugins = dist.package_data.get('ida_plugins', [])
        for plugin in ida_plugins:
            self.announce("[IDA PLUGIN INSTALL] copy plugin %s -> %s" % (plugin, install_dir), level=distutils.log.INFO)

            if not self.dry_run:
                self.copy_file(plugin,install_dir)

    def run(self, *args, **kwargs):
        """ Install ida plugins routine """
                
        dist = self.distribution  # type: setuptools.dist.Distribution

        # Custom install script
        if self.ida:
            install_dir = self.root # respect user-override install dir
            if not install_dir:     # otherwise return the ida install dir
                install_dir = IDA_INSTALL_DIRS[sys.platform](self.ida_version)
        
            if self.ida_install_deps:
                self.install_dependencies(dist, install_dir)

            self.install_packages(dist, install_dir)
            self.install_plugins(dist, install_dir)

        install.run(self)